"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@umijs/utils");
const path_1 = __importDefault(require("path"));
const constants_1 = require("../../constants");
const utils_2 = require("../utils");
const bundler = (0, utils_1.importLazy)(path_1.default.dirname(require.resolve('@umijs/bundler-webpack/package.json')));
exports.default = async (opts) => {
    const enableCache = process.env.FATHER_CACHE !== 'none';
    for (const config of opts.configProvider.configs) {
        utils_1.logger.info(`Bundle from ${utils_1.chalk.yellow(config.entry)} to ${utils_1.chalk.yellow(path_1.default.join(config.output.path, config.output.filename))}`);
        const { plugins: extraPostCSSPlugins, ...postcssLoader } = config.postcssOptions || {};
        await bundler.build({
            cwd: opts.cwd,
            config: {
                alias: config.alias,
                autoprefixer: config.autoprefixer,
                chainWebpack: config.chainWebpack,
                define: config.define,
                externals: config.externals,
                outputPath: config.output.path,
                // postcss config
                extraPostCSSPlugins,
                postcssLoader,
                // compatible with IE11 by default
                userConfig: {
                    targets: { ie: 11 },
                    jsMinifier: 'terser',
                    cssMinifier: 'cssnano',
                },
            },
            entry: {
                [path_1.default.parse(config.output.filename).name]: path_1.default.join(opts.cwd, config.entry),
            },
            babelPreset: [
                require.resolve('@umijs/babel-preset-umi'),
                {
                    presetEnv: {},
                    presetReact: (0, utils_2.getBabelPresetReactOpts)(opts.configProvider.pkg),
                    presetTypeScript: {},
                    pluginTransformRuntime: {},
                    pluginLockCoreJS: {},
                    pluginDynamicImportNode: false,
                },
            ],
            extraBabelPresets: config.extraBabelPresets,
            extraBabelPlugins: config.extraBabelPlugins,
            // configure library related options
            chainWebpack(memo) {
                memo.output.libraryTarget('umd');
                // modify webpack target
                if (config.platform === 'node') {
                    memo.target('node');
                }
                if (enableCache) {
                    // use father version as cache version
                    memo.merge({
                        cache: { version: require('../../../package.json').version },
                    });
                }
                return memo;
            },
            // enable webpack persistent cache
            ...(enableCache
                ? {
                    cache: {
                        cacheDirectory: path_1.default.join(opts.cwd, constants_1.CACHE_PATH, 'bundle-webpack'),
                    },
                }
                : {}),
        });
    }
};
